
#include "Arduino.h"
#include "nrf.h"
#include "mbed.h"

#define START_OUT 3 // commande de l'interrupteur
#define COMP_IN 4 // sortie du comparateur
#define START_IN 2 // entrée reliée à D2
#define PPI_CAPTURE_CH 10
#define PPI_START_CH 11

#define VREF 4.943 // mesurée
#define VCOMP 8.00 // mesurée
#define R 986e3 // mesurée

float time2C;

float mesure(uint16_t prescaler) {
  const uint8_t pins[14] = {0,0,11,12,15,13,14,23,21,27,2,1,8,13};
  const uint8_t ports[14] = {0,0,1,1,1,1,1,0,0,0,1,1,1,0};
  float timer_dt;
  NRF_GPIOTE->CONFIG[0] = 1 | (pins[COMP_IN]<<8) | (ports[COMP_IN]<<13) | (1<<16); // évènement rise sur COMP_IN
  NRF_GPIOTE->CONFIG[1] = 1 | (pins[START_IN]<<8) | (ports[START_IN]<<13) | (1<<16); // évènement rise sur START_IN
  NRF_TIMER4->TASKS_STOP = 1;
  NRF_TIMER4->MODE = 0; // timer mode
  NRF_TIMER4->BITMODE = 3; // 32 bits
  NRF_TIMER4->PRESCALER = prescaler;
  timer_dt = 1.0/16*pow(2,prescaler);
  NRF_TIMER4->TASKS_CLEAR = 1;
  NRF_PPI->CH[PPI_CAPTURE_CH].EEP = (uint32_t)&NRF_GPIOTE->EVENTS_IN[0];
  NRF_PPI->CH[PPI_CAPTURE_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_CAPTURE[0];
  NRF_PPI->FORK[PPI_CAPTURE_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_CLEAR;
  NRF_PPI->CHEN |= (1<< ((uint32_t)PPI_CAPTURE_CH));

  NRF_PPI->CH[PPI_START_CH].EEP = (uint32_t)&NRF_GPIOTE->EVENTS_IN[1];
  NRF_PPI->CH[PPI_START_CH].TEP = (uint32_t)&NRF_TIMER4->TASKS_START;
  NRF_PPI->CHEN |= (1<< ((uint32_t)PPI_START_CH));
  
  NRF_P1->DIR |= (1<<pins[START_OUT]);  //output
  NRF_P1->OUT |= (1 << pins[START_OUT]); // on ouvre l'interrupteur : la charge commence et le TIMER se déclenche
  
  while (NRF_GPIOTE->EVENTS_IN[0] == 0) {;} // on attend le front montant sur D4
  NRF_GPIOTE->EVENTS_IN[0] = 0;
  NRF_TIMER4->TASKS_STOP = 1;
  NRF_P1->OUT &= ~(1 << pins[START_OUT]); // on ferme l'interrupteur pour la décharge
  return timer_dt*NRF_TIMER4->CC[0];
}

void setup() {
  Serial.begin(115200);
  pinMode(START_OUT,OUTPUT);
  pinMode(COMP_IN,INPUT);
  pinMode(START_IN,INPUT);
  digitalWrite(START_OUT,LOW); // interrupteur fermé
  time2C = VREF/(R*VCOMP)*1e3;
}

void loop() {
  float temps;
  float C;
  temps = mesure(0);
  Serial.print(temps); Serial.print(" muS, ");
  C = temps*time2C;
  Serial.print(C,3); Serial.println(" nF");
  rtos::ThisThread::sleep_for(1000);

}   	       
    	       